@@ -13,17 +13,38 @@
  *		  1997 Matthias Prinke <m.prinke@trashcan.mcnet.de>
  *
  * The connections are:
- * PCF8574AP	  LCD
- * P0 (4)	  D4 (11)
- * P1 (5)	  D5 (12)
- * P2 (6)	  D6 (13)
- * P3 (7)	  D7 (14)
- * P4 (9)	  RS (4)
- * P5 (10)	  RW (5)
- * P6 (11)	  EN (6)
+ * PCF8574AP  Bit	LCD
+ * P0 (4)	  0x01  D4 (11)
+ * P1 (5)	  0x02  D5 (12)
+ * P2 (6)	  0x04  D6 (13)
+ * P3 (7)	  0x08  D7 (14)
+ * P4 (9)	  0x10  RS (4)
+ * P5 (10)	  0x20  RW (5)
+ * P6 (11)	  0x40  EN (6)
+ * P7 (12)	  0x80  /backlight (optional, active-low)
  *
- * Backlight
- * P7 (12)	  /backlight (optional, active-low)
+ * Configuration:
+ \verbatim
+   alternative wiring example:
+   PCF8574AP: P0 P1 P2 P3 P4 P5 P6 P7
+              |  |  |  |  |  |  |  |
+   HD44780:   RS RW EN BL D4 D5 D6 D7
+   
+   in LCDd.conf we then need to define
+    i2c_line_RS=0x01
+    i2c_line_RW=0x02
+    i2c_line_EN=0x04
+    i2c_line_BL=0x80
+    i2c_line_D4=0x10
+    i2c_line_D5=0x20
+    i2c_line_D6=0x40
+    i2c_line_D7=0x80
+    Backlight=yes
+    BacklightInvert=yes
+
+	The Backlight Invert is used if a 0 turns the backlight on, and 1 turns it off, i.e. npn transistor
+\endverbatim
+
  *
  * Configuration:
  * device=/dev/i2c-0   # the device file of the i2c bus
@@ -61,19 +82,8 @@
 #include <string.h>
 #include <errno.h>
 #include <unistd.h>
-#include <fcntl.h>
-#include <sys/ioctl.h>
-
-#ifdef HAVE_DEV_IICBUS_IIC_H
-#include <dev/iicbus/iic.h>
-#else /* HAVE_LINUX_I2C_DEV_H */
-#include <linux/i2c-dev.h>
-/* I2C_SLAVE is missing in linux/i2c-dev.h from kernel headers of 2.4.x kernels */
-#ifndef I2C_SLAVE
-#define I2C_SLAVE 0x0703  /* Change slave address */
-#endif
-#endif
 
+#include "i2c.h"
 
 // Generally, any function that accesses the LCD control lines needs to be
 // implemented separately for each HW design. This is typically (but not
@@ -89,6 +99,11 @@ void i2c_HD44780_close(PrivateData *p);
 #define RW	0x20
 #define EN	0x40
 #define BL	0x80
+#define D4	0x01
+#define D5	0x02
+#define D6	0x04
+#define D7	0x08
+#define BL_INVERT  0
 // note that the above bits are all meant for the data port of PCF8574
 
 #define I2C_ADDR_MASK 0x7f
@@ -97,13 +112,9 @@ void i2c_HD44780_close(PrivateData *p);
 static void
 i2c_out(PrivateData *p, unsigned char val)
 {
-	char data[2];
+	unsigned char data[2];
 	int datalen;
 	static int no_more_errormsgs=0;
-#ifdef HAVE_DEV_IICBUS_IIC_H
-	struct iiccmd cmd;
-	bzero(&cmd, sizeof(cmd));
-#endif
 
 	if (p->port & I2C_PCAX_MASK) { // we have a PCA9554 or similar, that needs a 2-byte command
 		data[0]=1; // command: read/write output port register
@@ -114,24 +125,13 @@ i2c_out(PrivateData *p, unsigned char val)
 		datalen=1;
 	}
 
-#ifdef HAVE_DEV_IICBUS_IIC_H
-	cmd.slave = (p->port & I2C_ADDR_MASK) << 1;
-	cmd.last = 1;
-	cmd.count = datalen;
-	cmd.buf = data;
-
-	if (ioctl(p->fd, I2CWRITE, &cmd) < 0) {
-#else /* HAVE_LINUX_I2C_DEV_H */
-	if (write(p->fd,data,datalen) != datalen) {
-#endif
-		p->hd44780_functions->drv_report(no_more_errormsgs?RPT_DEBUG:RPT_ERR, "HD44780: I2C: i2c write data %u to address 0x%02X failed: %s",
-			val, p->port & I2C_ADDR_MASK, strerror(errno));
+	if (i2c_write(p->i2c, data, datalen) < 0) {
+		p->hd44780_functions->drv_report(no_more_errormsgs?RPT_DEBUG:RPT_ERR, "HD44780: I2C: i2c write data %u failed: %s",
+			val, strerror(errno));
 		no_more_errormsgs=1;
 	}
 }
 
-#define DEFAULT_DEVICE		"/dev/i2c-0"
-
 
 /**
  * Initialize the driver.
@@ -144,61 +144,60 @@ hd_init_i2c(Driver *drvthis)
 {
 	PrivateData *p = (PrivateData*) drvthis->private_data;
 	HD44780_functions *hd44780_functions = p->hd44780_functions;
+	char device[256] = I2C_DEFAULT_DEVICE;
 
-	int enableLines = EN;
-	char device[256] = DEFAULT_DEVICE;
+	p->i2c_backlight_invert = drvthis->config_get_bool(drvthis->name, "BacklightInvert", 0, BL_INVERT);
+	p->i2c_line_RS = drvthis->config_get_int(drvthis->name, "i2c_line_RS", 0, RS);
+	p->i2c_line_RW = drvthis->config_get_int(drvthis->name, "i2c_line_RW", 0, RW);
+	p->i2c_line_EN = drvthis->config_get_int(drvthis->name, "i2c_line_EN", 0, EN);
+	p->i2c_line_BL = drvthis->config_get_int(drvthis->name, "i2c_line_BL", 0, BL);
+	p->i2c_line_D4 = drvthis->config_get_int(drvthis->name, "i2c_line_D4", 0, D4);
+	p->i2c_line_D5 = drvthis->config_get_int(drvthis->name, "i2c_line_D5", 0, D5);
+	p->i2c_line_D6 = drvthis->config_get_int(drvthis->name, "i2c_line_D6", 0, D6);
+	p->i2c_line_D7 = drvthis->config_get_int(drvthis->name, "i2c_line_D7", 0, D7);
+	
+	report(RPT_INFO, "HD44780: I2C: Init using D4 and D5, and or'd lines, invert", p->i2c_line_RS);
+	report(RPT_INFO, "HD44780: I2C: Pin RS mapped to 0x%02X", p->i2c_line_RS);
+	report(RPT_INFO, "HD44780: I2C: Pin RW mapped to 0x%02X", p->i2c_line_RW);
+	report(RPT_INFO, "HD44780: I2C: Pin EN mapped to 0x%02X", p->i2c_line_EN);
+	report(RPT_INFO, "HD44780: I2C: Pin BL mapped to 0x%02X", p->i2c_line_BL);
+	report(RPT_INFO, "HD44780: I2C: Pin D4 mapped to 0x%02X", p->i2c_line_D4);
+	report(RPT_INFO, "HD44780: I2C: Pin D5 mapped to 0x%02X", p->i2c_line_D5);
+	report(RPT_INFO, "HD44780: I2C: Pin D6 mapped to 0x%02X", p->i2c_line_D6);
+	report(RPT_INFO, "HD44780: I2C: Pin D7 mapped to 0x%02X", p->i2c_line_D7);
+	report(RPT_INFO, "HD44780: I2C: Invert Backlight %d", p->i2c_backlight_invert);
+	
 #ifdef HAVE_DEV_IICBUS_IIC_H
 	struct iiccmd cmd;
 	bzero(&cmd, sizeof(cmd));
 #endif
 
-	p->backlight_bit = BL;
+	p->backlight_bit = p->i2c_line_BL;
 
 	/* READ CONFIG FILE */
 
 	/* Get serial device to use */
-	strncpy(device, drvthis->config_get_string(drvthis->name, "Device", 0, DEFAULT_DEVICE), sizeof(device));
+	strncpy(device, drvthis->config_get_string(drvthis->name, "Device", 0, I2C_DEFAULT_DEVICE), sizeof(device));
 	device[sizeof(device)-1] = '\0';
 	report(RPT_INFO,"HD44780: I2C: Using device '%s' and address 0x%02X for a %s",
 		device, p->port & I2C_ADDR_MASK, (p->port & I2C_PCAX_MASK) ? "PCA9554(A)" : "PCF8574(A)");
 
-	// Open the I2C device
-	p->fd = open(device, O_RDWR);
-	if (p->fd < 0) {
-		report(RPT_ERR, "HD44780: I2C: open i2c device '%s' failed: %s", device, strerror(errno));
+	p->i2c = i2c_open(device, p->port & I2C_ADDR_MASK);
+	if (!p->i2c) {
+		report(RPT_ERR, "HD44780: I2C: connecting to device '%s' slave 0x%02X failed:", device, p->port & I2C_ADDR_MASK, strerror(errno));
 		return(-1);
 	}
 
-	// Set I2C address
-#ifdef HAVE_DEV_IICBUS_IIC_H
-	cmd.slave = (p->port & I2C_ADDR_MASK) << 1;
-	cmd.last = 0;
-	cmd.count = 0;
-	if (ioctl(p->fd, I2CRSTCARD, &cmd) < 0) {
-		report(RPT_ERR, "HD44780: I2C: reset bus failed: %s", strerror(errno));
-		return -1;
-	}
-	if (ioctl(p->fd, I2CSTART, &cmd) < 0) {
-		report(RPT_ERR, "HD44780: I2C: set address to 0x%02X: %s", p->port & I2C_ADDR_MASK, strerror(errno));
-		return -1;
-	}
-#else /* HAVE_LINUX_I2C_DEV_H */
-	if (ioctl(p->fd,I2C_SLAVE, p->port & I2C_ADDR_MASK) < 0) {
-		report(RPT_ERR, "HD44780: I2C: set address to '%i': %s", p->port & I2C_ADDR_MASK, strerror(errno));
-		return(-1);
-	}
-#endif
-
 	if (p->port & I2C_PCAX_MASK) { // we have a PCA9554 or similar, that needs special config
-		char data[2];
+		unsigned char data[2];
 		data[0] = 2; // command: set polarity inversion
 		data[1] = 0; // -> no polarity inversion
-		if (write(p->fd,data,2) != 2) {
+		if (i2c_write(p->i2c, data, 2) < 0) {
 			report(RPT_ERR, "HD44780: I2C: i2c set polarity inversion failed: %s", strerror(errno));
 		}
 		data[0] = 3; // command: set output direction
 		data[1] = 0; // -> all pins are outputs
-		if (write(p->fd,data,2) != 2) {
+		if (i2c_write(p->i2c, data, 2) <0) {
 			report(RPT_ERR, "HD44780: I2C: i2c set output direction failed: %s", strerror(errno));
 		}
 	}
@@ -209,44 +208,49 @@ hd_init_i2c(Driver *drvthis)
 
 	// powerup the lcd now
 	/* We'll now send 0x03 a couple of times,
-	 * which is in fact (FUNCSET | IF_8BIT) >> 4 */
-	i2c_out(p, 0x03);
+	 * which is in fact (FUNCSET | IF_8BIT) >> 4
+	 */
+	i2c_out(p, p->i2c_line_D4 | p->i2c_line_D5);
 	if (p->delayBus)
 		hd44780_functions->uPause(p, 1);
 
-	i2c_out(p, enableLines | 0x03);
+	i2c_out(p, p->i2c_line_EN | p->i2c_line_D4 | p->i2c_line_D5);
 	if (p->delayBus)
 		hd44780_functions->uPause(p, 1);
-	i2c_out(p, 0x03);
+	i2c_out(p, p->i2c_line_D4 | p->i2c_line_D5);
 	hd44780_functions->uPause(p, 15000);
 
-	i2c_out(p, enableLines | 0x03);
+	i2c_out(p, p->i2c_line_EN | p->i2c_line_D4 | p->i2c_line_D5);
 	if (p->delayBus)
 		hd44780_functions->uPause(p, 1);
-	i2c_out(p, 0x03);
+	i2c_out(p, p->i2c_line_D4 | p->i2c_line_D5);
 	hd44780_functions->uPause(p, 5000);
 
-	i2c_out(p, enableLines | 0x03);
+	i2c_out(p, p->i2c_line_EN | p->i2c_line_D4 | p->i2c_line_D5);
 	if (p->delayBus)
 		hd44780_functions->uPause(p, 1);
-	i2c_out(p, 0x03);
+	i2c_out(p, p->i2c_line_D4 | p->i2c_line_D5);
 	hd44780_functions->uPause(p, 100);
 
-	i2c_out(p, enableLines | 0x03);
+	i2c_out(p, p->i2c_line_EN | p->i2c_line_D4 | p->i2c_line_D5);
 	if (p->delayBus)
 		hd44780_functions->uPause(p, 1);
-	i2c_out(p, 0x03);
+	i2c_out(p, p->i2c_line_D4 | p->i2c_line_D5);
 	hd44780_functions->uPause(p, 100);
 
 	// now in 8-bit mode...  set 4-bit mode
-	i2c_out(p, 0x02);
+	/*
+	OLD   (FUNCSET | IF_4BIT) >> 4 0x02
+	ALT   (FUNCSET | IF_4BIT)      0x20
+	*/
+	i2c_out(p, p->i2c_line_D5);
 	if (p->delayBus)
 		hd44780_functions->uPause(p, 1);
 
-	i2c_out(p, enableLines | 0x02);
+	i2c_out(p, p->i2c_line_EN | p->i2c_line_D5);
 	if (p->delayBus)
 		hd44780_functions->uPause(p, 1);
-	i2c_out(p, 0x02);
+	i2c_out(p, p->i2c_line_D5);
 	hd44780_functions->uPause(p, 100);
 
 	// Set up two-line, small character (5x8) mode
@@ -260,12 +264,8 @@ hd_init_i2c(Driver *drvthis)
 
 void
 i2c_HD44780_close(PrivateData *p) {
-	if (p->fd >= 0) {
-#ifdef HAVE_DEV_IICBUS_IIC_H
-		ioctl(p->fd, I2CSTOP);
-#endif
-		close(p->fd);
-	}
+	if (p->i2c >= 0)
+		i2c_close(p->i2c);
 }
 
 
@@ -279,23 +279,28 @@ i2c_HD44780_close(PrivateData *p) {
 void
 i2c_HD44780_senddata(PrivateData *p, unsigned char displayID, unsigned char flags, unsigned char ch)
 {
-	unsigned char enableLines = 0, portControl = 0;
-	unsigned char h = (ch >> 4) & 0x0f;     // high and low nibbles
-	unsigned char l = ch & 0x0f;
-
+	unsigned char portControl = 0;
+	unsigned char h=0;
+	unsigned char l=0;
+	if( ch & 0x80 ) h |= p->i2c_line_D7;
+	if( ch & 0x40 ) h |= p->i2c_line_D6;
+	if( ch & 0x20 ) h |= p->i2c_line_D5;
+	if( ch & 0x10 ) h |= p->i2c_line_D4;
+	if( ch & 0x08 ) l |= p->i2c_line_D7;
+	if( ch & 0x04 ) l |= p->i2c_line_D6;
+	if( ch & 0x02 ) l |= p->i2c_line_D5;
+	if( ch & 0x01 ) l |= p->i2c_line_D4;
 	if (flags == RS_INSTR)
 		portControl = 0;
 	else //if (flags == RS_DATA)
-		portControl = RS;
+		portControl = p->i2c_line_RS;
 
 	portControl |= p->backlight_bit;
 
-	enableLines = EN;
-
 	i2c_out(p, portControl | h);
 	if (p->delayBus)
 		p->hd44780_functions->uPause(p, 1);
-	i2c_out(p, enableLines | portControl | h);
+	i2c_out(p, p->i2c_line_EN | portControl | h);
 	if (p->delayBus)
 		p->hd44780_functions->uPause(p, 1);
 	i2c_out(p, portControl | h);
@@ -303,7 +308,7 @@ i2c_HD44780_senddata(PrivateData *p, unsigned char displayID, unsigned char flag
 	i2c_out(p, portControl | l);
 	if (p->delayBus)
 		p->hd44780_functions->uPause(p, 1);
-	i2c_out(p, enableLines | portControl | l);
+	i2c_out(p, p->i2c_line_EN | portControl | l);
 	if (p->delayBus)
 		p->hd44780_functions->uPause(p, 1);
 	i2c_out(p, portControl | l);
@@ -317,7 +322,9 @@ i2c_HD44780_senddata(PrivateData *p, unsigned char displayID, unsigned char flag
  */
 void i2c_HD44780_backlight(PrivateData *p, unsigned char state)
 {
-	p->backlight_bit = ((!p->have_backlight||state) ? 0 : BL);
-
+	if ( p->i2c_backlight_invert == 0 )
+		p->backlight_bit = ((!p->have_backlight||state) ? 0 : p->i2c_line_BL);
+	else // Inverted backlight - npn transistor
+		p->backlight_bit = ((p->have_backlight && state) ? p->i2c_line_BL : 0);
 	i2c_out(p, p->backlight_bit);
 }
