@@ -29,6 +29,8 @@
 # include <ftdi.h>
 #endif
 
+#include "i2c.h"
+
 /** \name Symbolic names for connection types
  *@{*/
 #define HD44780_CT_UNKNOWN		0
@@ -57,6 +59,8 @@
 #define HD44780_CT_PIPLATE		23
 #define HD44780_CT_SPI			24
 #define HD44780_CT_PIFACECAD		25
+#define HD44780_CT_LCM162		26
+#define HD44780_CT_GPIO			27
 /**@}*/
 
 /** \name Symbolic names for interface types
@@ -155,6 +159,22 @@ typedef struct hd44780_private_data {
 	int ftdi_line_backlight;
 #endif
 
+#ifdef HAVE_I2C
+	/* i2c based connection types */
+	
+	int i2c_backlight_invert;
+	int i2c_line_RS;
+	int i2c_line_RW;
+	int i2c_line_EN;
+	int i2c_line_BL;
+	int i2c_line_D4;
+	int i2c_line_D5;
+	int i2c_line_D6;
+	int i2c_line_D7;
+
+	I2CHandle *i2c;
+#endif
+
 #ifdef WITH_ETHLCD
 	int sock;		/**< socket for TCP devices */
 #endif
@@ -176,6 +196,7 @@ typedef struct hd44780_private_data {
 	/* Connection type data */
 	int connectiontype;
 	struct hwDependentFns *hd44780_functions;
+	void *connection_data;
 
 	/** \name Screens spanning several display
 	 *@{*/
