@@ -1,8 +1,6 @@
 #ifndef STV5730_H
 #define STV5730_H
 
-#include "lcd.h"
-
 MODULE_EXPORT int stv5730_init (Driver *drvthis);
 MODULE_EXPORT void stv5730_close (Driver *drvthis);
 MODULE_EXPORT int stv5730_width (Driver *drvthis);
@@ -13,9 +11,9 @@ MODULE_EXPORT void stv5730_clear (Driver *drvthis);
 MODULE_EXPORT void stv5730_flush (Driver *drvthis);
 MODULE_EXPORT void stv5730_string (Driver *drvthis, int x, int y, const char string[]);
 MODULE_EXPORT void stv5730_chr (Driver *drvthis, int x, int y, char c);
-MODULE_EXPORT void stv5730_old_vbar (Driver *drvthis, int x, int len);
-MODULE_EXPORT void stv5730_old_hbar (Driver *drvthis, int x, int y, int len);
+MODULE_EXPORT void stv5730_vbar(Driver *drvthis, int x, int y, int len, int promille, int options);
+MODULE_EXPORT void stv5730_hbar (Driver *drvthis, int x, int y, int len, int promille, int options);
 MODULE_EXPORT void stv5730_num (Driver *drvthis, int x, int num);
-MODULE_EXPORT void stv5730_old_icon (Driver *drvthis, int which, char dest);
+MODULE_EXPORT int  stv5730_icon(Driver *drvthis, int x, int y, int icon);
 
 #endif
